require 'open-uri'
require 'nokogiri'
require 'json'
require 'net/http'

require_relative 'Notificator'
require_relative 'IndexWriter'
require_relative 'Dbops'
require_relative 'Trackable'

module DailyRep
  class IBrowser
    include Dbops
    include Trackable
    attr_reader :entity

    @@notificators = Notificator.get_clients
    @@index_writer = IndexWriter.new
    @@app_cycle_methods = ['process', 'write_to_db', 'web_reload', 'notify']

    def initialize *inputs
      @entity = self.class.to_s.downcase.split('::').last
    end

    def push_note note
      @@notificators.each { |client|
        client.push_note(note)
      }
      # @@client.push_note(nil, @@note_title,  note)
    end

    def set_html entity, values, push=0
      @@index_writer.set_html entity, values, push
    end

    def set_html_with_diff entity, scope, diff, reset=false
      @@index_writer.set_html_with_diff entity, scope, diff, reset
    end

    def self.write_html
      @@index_writer.write_html
    end


    def get_source_http source, val_scope
      page = Nokogiri::HTML(open(source.to_s))
      page.css(val_scope)
    end

    def get_source_json source
      uri = URI(source)
      out_json = Net::HTTP.get(uri)
      JSON.parse(out_json)
    end

    def method_missing(meth, *args, &block)
      if @@app_cycle_methods.include?(meth.to_s)
        begin
          self.log_msg meth.to_s
          block.call
          rescue Exception => e
            log_error e
        end
      else
        super
      end
    end

    def run
      self.process
      self.write_to_db if Configer.is_phase_enabled?('write_to_db')
      self.notify if Configer.is_phase_enabled?('notify')
      self.web_reload if Configer.is_phase_enabled?('web_reload')
    end

  end
end
