
class String
  def to_bool
    return true if self == true || self =~ (/(true|t|yes|y|1)$/i)
    return false if self == false || self.blank? || self =~ (/(false|f|no|n|0)$/i)
    raise ArgumentError.new("invalid value for Boolean: \"#{self}\"")
  end
end

ActiveRecord::Base.establish_connection(
  :adapter  => 'sqlite3',
  :database => DATABASE,
  :pool     => 5,
  :timeout  => 5000
)

def generate_database
  ActiveRecord::Schema.define(version: 201502161646) do
    create_table "Histories", force: false do |t|
      t.text     "entity"
      t.text     "parameter"
      t.text     "val"
      t.datetime "created_at"
      t.datetime "notified"
    end
  end
end

tables = ActiveRecord::Base.connection.execute(
    "SELECT name FROM sqlite_master WHERE type='table' and name = 'Histories'; "
)

if tables.empty?
  generate_database
else
  generate_database unless tables[0]["name"] == 'Histories'
end


class History < ActiveRecord::Base
end

module DailyRep

  START_TIME = Time.now

  #constants
  P_GO_FILE_PATH    = 'GoFilePath'
  P_PUSH_CLIENTS    = 'Push clients'
  P_PUSH_NOTE_TITLE = 'Push title'
  P_RAISE_ERRORS    = 'Raise errors'

  PA_ENTITIES   = 'Entities'
  PA_APP_CYCLE  = 'app_cycle'
  P_INDEX_PATH  = 'Index path'
  P_NOTIF_DELTA = 'notification delta'
  P_ENABLED     = 'enabled'
  P_TIME_FRAME  = 'time frame'
  P_SOURCE      = 'source'
  P_SEARCH_STRS = 'searchStr'





  class Configer
    def self.set_index_file file
      @@config_json[P_INDEX_PATH] = file
    end

    File.open(CONFIG_FILE) {|file| @@config_json = JSON.parse(file.read) }

    def self.index_path
      @@config_json[P_INDEX_PATH]
    end

    def self.push_note_title
      @@config_json[P_PUSH_NOTE_TITLE]
    end

    def self.raise_error?
      @@config_json[P_RAISE_ERRORS].to_bool
    end

    def self.exctract_entity_params entity, parameter_name
      @@config_json[PA_ENTITIES][entity][parameter_name]
    end

    def self.is_phase_enabled? phase
      @@config_json[PA_APP_CYCLE][phase].to_bool
    end
    #+
    def self.is_entity_enabled? entity
      time_arr = @@config_json[PA_ENTITIES][entity][P_TIME_FRAME].split('..').map { |el|
        el.to_i
      }
      (time_arr[0]..time_arr[1]).include?(Time.now.hour) &&
        @@config_json[PA_ENTITIES][entity][P_ENABLED].to_bool
    end
    #+
    def self.enabled_entities
      @@config_json[PA_ENTITIES].keys.map { |entity|
        entity.downcase.capitalize if is_entity_enabled?(entity)
      }
    end
    #+
    def self.get_client_tokens
      @@config_json[P_PUSH_CLIENTS].values
    end

    # def self.include_search_strs? entity
    #   result = false
    #   result = true if @@config_json[PA_ENTITIES][entity][P_SEARCH_STRS]
    #   return result
    # end

    def self.get_search_strs entity
      @@config_json[PA_ENTITIES][entity][P_SEARCH_STRS]
    end

    def self.method_missing(meth, *args, &block)
      method_inputs = meth.to_s.split('_')
      out_param = ''
      @@config_json[PA_ENTITIES].keys.each { |key|
        if method_inputs[0] == key.downcase
          out_param = self.exctract_entity_params method_inputs[0], method_inputs[1]
        end
      }
      super if not @@config_json[PA_ENTITIES].keys.include?(method_inputs[0])
      return out_param
    end

  end

end
